'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var loadable = _interopDefault(require('@loadable/component'));
var tslib_1 = require('tslib');
var fp = require('lodash/fp');
var __chunk_1 = require('./chunk.js');
var router = require('@reach/router');
var capitalize = _interopDefault(require('capitalize'));
require('fast-deep-equal');
require('array-sort');
require('ulid');
require('match-sorter');
var tag = require('@mdx-js/tag');

const BasePlayground = loadable(() => Promise.resolve(require('./chunk2.js')));
const Playground = props => typeof window !== 'undefined' ? (React.createElement(React.Suspense, { fallback: null },
    React.createElement(BasePlayground, Object.assign({}, props)))) : null;

const AsyncComponent = defaultProps => {
    const [loading, setLoading] = React.useState(true);
    const [error, setError] = React.useState(null);
    const [data, setData] = React.useState({});
    React.useEffect(() => {
        const { getInitialProps } = defaultProps;
        if (getInitialProps && __chunk_1.isFn(getInitialProps)) {
            setLoading(true);
            getInitialProps(defaultProps)
                .then((data) => {
                setLoading(false);
                setError(null);
                setData(data);
            })
                .catch((err) => {
                setLoading(false);
                setError(err);
                setData({});
            });
        }
    }, []);
    const { as: Comp, getInitialProps } = defaultProps, props = tslib_1.__rest(defaultProps, ["as", "getInitialProps"]);
    return React.createElement(Comp, Object.assign({}, props, { data: Object.assign({}, data, { loading, error }) }));
};

const loadRoute = (path, imports) => {
    return loadable(async () => {
        const importFn = fp.get(path, imports);
        const { default: Component, getInitialProps } = await importFn();
        const ExportedComponent = props => (React.createElement(AsyncComponent, Object.assign({}, props, { as: Component || 'div', getInitialProps: getInitialProps })));
        return ExportedComponent;
    });
};
const AsyncRoute = defaultProps => {
    const { asyncComponent, path, entry } = defaultProps, routeProps = tslib_1.__rest(defaultProps, ["asyncComponent", "path", "entry"]);
    const components = __chunk_1.useComponents();
    const Page = components.page;
    const Component = asyncComponent;
    const props = Object.assign({}, routeProps, { doc: entry });
    return Page ? (React.createElement(Page, Object.assign({}, props),
        React.createElement(Component, Object.assign({}, props)))) : (React.createElement(Component, Object.assign({}, props)));
};

const Link = React.forwardRef((props, ref) => {
    const isActive = React.useCallback(({ isCurrent }) => {
        return isCurrent ? { className: `${props.className} active` } : {};
    }, [props.className]);
    return React.createElement(router.Link, Object.assign({}, props, { getProps: isActive, ref: ref }));
});
Link.displayName = 'Link';

const RE_OBJECTOF = /(?:React\.)?(?:PropTypes\.)?objectOf\((?:React\.)?(?:PropTypes\.)?(\w+)\)/;
const getTypeStr = (type) => {
    switch (type.name.toLowerCase()) {
        case 'instanceof':
            return `Class(${type.value})`;
        case 'enum':
            if (type.computed)
                return type.value;
            return type.value
                ? type.value.map((v) => `${v.value}`).join(' │ ')
                : type.raw;
        case 'union':
            return type.value
                ? type.value.map((t) => `${getTypeStr(t)}`).join(' │ ')
                : type.raw;
        case 'array':
            return type.raw;
        case 'arrayof':
            return `Array<${getTypeStr(type.value)}>`;
        case 'custom':
            if (type.raw.indexOf('function') !== -1 || type.raw.indexOf('=>') !== -1)
                return 'Custom(Function)';
            else if (type.raw.toLowerCase().indexOf('objectof') !== -1) {
                const m = type.raw.match(RE_OBJECTOF);
                if (m && m[1])
                    return `ObjectOf(${capitalize(m[1])})`;
                return 'ObjectOf';
            }
            return 'Custom';
        case 'bool':
            return 'Boolean';
        case 'func':
            return 'Function';
        case 'shape':
            const shape = type.value;
            const rst = {};
            Object.keys(shape).forEach(key => {
                rst[key] = getTypeStr(shape[key]);
            });
            return JSON.stringify(rst, null, 2);
        default:
            return capitalize(type.name);
    }
};
const humanize = (type) => getTypeStr(type);

const getPropType = (prop) => {
    const propName = fp.get('name', prop.flowType || prop.type);
    if (!propName)
        return null;
    const isEnum = propName.startsWith('"') || propName === 'enum';
    const name = capitalize(isEnum ? 'enum' : propName);
    const value = fp.get('type.value', prop);
    if (!name)
        return null;
    if ((isEnum && typeof value === 'string') ||
        (!prop.flowType && !isEnum && !value) ||
        (prop.flowType && !prop.flowType.elements)) {
        return name;
    }
    return prop.flowType ? humanize(prop.flowType) : humanize(prop.type);
};
const Props = ({ of: component }) => {
    const components = __chunk_1.useComponents();
    const { props: stateProps } = React.useContext(__chunk_1.doczState.context);
    const filename = fp.get('__filemeta.filename', component);
    const found = stateProps &&
        stateProps.length > 0 &&
        stateProps.find(item => item.key === filename);
    const definition = fp.last(found ? found.value : []);
    const props = fp.get('props', definition) || [];
    if (!props)
        return null;
    if (!components.props)
        return null;
    return React.createElement(components.props, { props: props, getPropType: getPropType });
};

const goToHash = ({ location }) => {
    setTimeout(() => {
        if (location && location.hash) {
            const id = location.hash.substring(1);
            const el = document.getElementById(id);
            if (el)
                el.scrollIntoView();
        }
    });
};
const Routes = ({ imports }) => {
    const components = __chunk_1.useComponents();
    const { entries } = React.useContext(__chunk_1.doczState.context);
    const NotFound = components.notFound;
    const Loading = components.loading;
    const history = React.useMemo(() => router.createHistory(window), []);
    React.useEffect(() => {
        history.listen(goToHash);
    }, []);
    return (React.createElement(tag.MDXProvider, { components: components },
        React.createElement(router.LocationProvider, { history: history },
            React.createElement(React.Suspense, { fallback: React.createElement(Loading, null) },
                React.createElement(router.Router, { basepath: DOCZ_BASE_URL },
                    React.createElement(NotFound, { default: true }),
                    entries &&
                        entries.map(({ key: path, value: entry }) => {
                            const props = { path, entries, components };
                            const component = loadRoute(path, imports);
                            return (React.createElement(AsyncRoute, Object.assign({}, props, { entry: entry, key: entry.id, path: entry.route, asyncComponent: component })));
                        }))))));
};

function theme(themeConfig, transform = c => c) {
    return WrappedComponent => {
        const Theme = React.memo(props => {
            const { linkComponent } = props;
            const { db, children, wrapper: Wrapper = React.Fragment } = props;
            const initial = Object.assign({}, db, { themeConfig, transform, linkComponent });
            return (React.createElement(__chunk_1.doczState.Provider, { initial: initial },
                React.createElement(Wrapper, null,
                    React.createElement(WrappedComponent, null, children))));
        });
        return Theme;
    };
}

exports.doczState = __chunk_1.doczState;
exports.useConfig = __chunk_1.useConfig;
exports.useDataServer = __chunk_1.useDataServer;
exports.useDocs = __chunk_1.useDocs;
exports.useMenus = __chunk_1.useMenus;
exports.usePrevious = __chunk_1.usePrevious;
exports.useWindowSize = __chunk_1.useWindowSize;
exports.ComponentsProvider = __chunk_1.ComponentsProvider;
exports.useComponents = __chunk_1.useComponents;
exports.Playground = Playground;
exports.AsyncRoute = AsyncRoute;
exports.loadRoute = loadRoute;
exports.Link = Link;
exports.Props = Props;
exports.Routes = Routes;
exports.theme = theme;
